import os
from ctypes import *

lib_path = os.path.join(os.path.dirname(__file__), "lib.so")
lib = CDLL(lib_path)

class GenotypeNode(Structure):
    pass

class Person(Structure):
    pass

class Relation(Structure):
    pass

class GenotypeNode(Structure):
    """
    C linked list that holds arrays of (probilities of) genetic data.
    """
    _fields_ = [
        ("geneArray", POINTER(c_double)),
        ("nextCopy", POINTER(GenotypeNode))
    ]

class GenotypeData(Structure):
    """
    C structure that carries genetic metadata
    """
    _fields_ = [
        ("geneCount", c_int),
        ("alleleCount", c_int),
        ("genes", POINTER(GenotypeNode))
    ]



class Person(Structure):
    """
    C structure to hold a person
    """
    _fields_ = [
        ("gender", c_int),
        ("name", POINTER(c_char)),
        ("id", c_int),
        ("relationOfOrigin", POINTER(Relation)),
        ("geneticData", POINTER(GenotypeData)),
        ("prevSibling", POINTER(Person)),
        ("nextSibling", POINTER(Person)),
        ("firstRelation", POINTER(Relation))
    ]

class Relation(Structure):
    """
    C structure to hold a relation
    """
    _fields_ = [
        ("male", POINTER(Person)),
        ("female", POINTER(Person)),
        ("firstChild", POINTER(Person)),
        ("maleNextRelation", POINTER(Relation)),
        ("femaleNextRelation", POINTER(Relation))
    ]


lib.createPerson.argtypes = [POINTER(Relation), c_int, c_char_p, POINTER(GenotypeData)]
lib.createPerson.restype  = POINTER(Person)

lib.addPerson.argtypes = [c_char_p , c_char_p, c_int, c_char_p, POINTER(GenotypeData) ]
lib.addPerson.restype = None

lib.findPersonById.argtypes = [c_int]
lib.findPersonById.restype = POINTER(Person)

lib.findPersonByName.argtypes = [c_char_p]
lib.findPersonByName.restype = POINTER(Person)

lib.displayPerson.argtypes = [POINTER(Person)]
lib.displayPerson.restype = None

lib.getRelationType.argtypes = [POINTER(Person), POINTER(Person)]
lib.getRelationType.restype = c_int